/*
 * Copyright (c) 2017 TOYOTA MOTOR CORPORATION
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef QLIBWINDOWMANAGER_H
#define QLIBWINDOWMANAGER_H
#include <libwindowmanager.h>
#include <functional>
#include <QObject>
#include <string>

class QLibWindowmanager : public QObject{
Q_OBJECT
public:
    explicit QLibWindowmanager(QObject *parent = nullptr);
    ~QLibWindowmanager();

    QLibWindowmanager(const QLibWindowmanager &) = delete;
    QLibWindowmanager &operator=(const QLibWindowmanager &) = delete;

public:
    using handler_fun = std::function<void(json_object *object)>;

    enum QEventType {
       Event_Active = LibWindowmanager::Event_Active,
       Event_Inactive,

       Event_Visible,
       Event_Invisible,

       Event_SyncDraw,
       Event_FlushDraw,
    };

    int init(int port, const QString &token);

    // WM API
    Q_INVOKABLE int requestSurface(const QString &label);
    Q_INVOKABLE int activateSurface(const QString &label);
    Q_INVOKABLE int activateSurface(const QString &label, const QString &drawing_area);
    Q_INVOKABLE int deactivateSurface(const QString &label);
    Q_INVOKABLE int endDraw(const QString &label);
    void set_event_handler(enum QEventType et, handler_fun f);

public slots:
    void slotActivateSurface();

private:
    LibWindowmanager* wm;
    char* applabel;
    bool isActive;

};
#endif // LIBWINDOWMANAGER_H
